library("MASS") # we need this one for mvrnorm function (multivariate normal)

mcmc_regression=function(n_iter,y,x,init,hyper) {
# n_iter = number of iterations, y = data (n*1 vector), x = data (n*k matrix)
# init = initial values for sigma^2,beta
# hyper = hyperparameters (a, b).

# model for y, x
# y~ N(x*beta,sig2)
  
  n_obs=nrow(x) # number of observations in x, should be the same as length(y)
  
  
  # some precalculations
  inv_xtx=solve(t(x)%*%x) # (X'X)^(-1)
  b_ols=inv_xtx%*%t(x)%*%y # (X'X)^(-1)X'Y
  
  # start from initial values for sigma^2 and beta
  sig2=init[1]
  beta=init[2:length(init)]
    
  output=matrix(0,ncol=ncol(x)+1,nrow=n_iter) # output matrix, (k+1) parameters, n_iter iterations
  
  for(i in 1:n_iter)   {
    
    # update beta
    beta=mvrnorm(1,b_ols,sig2*inv_xtx) # generate beta according to multinormal
    
    # update sigma^2
    resid=y-x%*%beta
    sig2=1/rgamma(1,hyper[1]+n_obs/2,(hyper[2]+t(resid)%*%resid/2)) # generate sigma2
  
    output[i,]=c(sig2,beta)  } # save iteration
  output }

#
#    
#
data = read.csv("..\\Desktop\\MCMC\\andrey_data.csv")

#
# 1.  -     sigma^2, bi  
#
#          2000-2004
#  : sigma^2, b1,...,bk, h;  - .
store = matrix(NA, ncol=5, nrow=5)

for(i in 1:5) {
  reg = lm(SocCoh ~ Sec + QL, subset(data, Year==1999+i))
  sum = summary(reg)
  store[i,] = c(sum$sigma^2, reg$coefficients, 1/sum$sigma^2)
}

#        
means = apply(store, 2, mean)
sd = apply(store, 2, sd)

starting_values = means[-5]
hyper_values = c((means[5]/sd[5])^2, means[5]/(sd[5])^2)

#
# 2.  MCMC
#
#  
data.2005 = subset(data, Year==2005)
nobs=nrow(data.2005)
x_data=cbind(c(rep(1,nobs)), data.2005$Sec, data.2005$QL)
y_data=data.2005$SocCoh
niterations = 5000
res=mcmc_regression(niterations,y_data,x_data,starting_values,hyper_values)

#
# 3.        sigma^2
#
hist(res[100:niterations,4],main=" b3", xlab="b3", ylab="")
hist(res[100:niterations,3],main=" b2", xlab="b2", ylab="")
hist(res[100:niterations,2],main=" b1", xlab="b1", ylab="")
hist(res[100:niterations,1],main=" sigma^2", xlab="sigma^2", ylab="")

res_means = apply(res[100:niterations,], 2, mean)

#
# 4.     Y1, Y2
#
#   
x1 = c(1, 8.04, 7.59) #     - 
x2 = c(1, 2.86, 3.13) #   - 
forecast = res[,-1] %*% cbind(x1, x2)

#   (    )
hist(forecast[100:niterations,1],main="  Y1", xlab="Y1", ylab="")
hist(forecast[100:niterations,2],main="  Y2", xlab="Y2", ylab="")
forecast_means = apply(forecast[100:niterations,], 2, mean)

#
# 5.    ,    .
#
#    .   -  2
ngrid = 100
B = cbind(c(2.747, -0.053), c(-0.053, 2.673))
cr.x = seq(5, 9, length.out = ngrid)
cr.y = seq(1, 5, length.out = ngrid)
cr.z = matrix(NA, ngrid, ngrid)
for (i in 1:ngrid) {
  for (j in 1:ngrid) {
    cr.v = c(cr.x[i] - 7.03, cr.y[j] - 3.29)
    cr.z[i,j] = 0.5*t(cr.v) %*% B %*% cr.v
  }
}

#      .   
#    99%, 95%  90%
plot(forecast[100:niterations,], xlim=c(5.5,8.8), ylim=c(1.1,5.1),
     main="   ", xlab="Y1", ylab="Y2")
contour(cr.x,cr.y,cr.z, levels=c(2.695, 3.682, 6.359), add=TRUE)

# .
